package test;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Map;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;
import javax.xml.ws.Service.Mode;
import javax.xml.ws.soap.SOAPBinding;

import org.picketlink.identity.federation.core.wstrust.WSTrustConstants;
import org.picketlink.identity.federation.core.wstrust.WSTrustException;
import org.picketlink.identity.federation.core.wstrust.WSTrustJAXBFactory;
import org.picketlink.identity.federation.core.wstrust.plugins.saml.SAMLUtil;
import org.picketlink.identity.federation.core.wstrust.wrappers.RequestSecurityToken;
import org.picketlink.identity.federation.core.wstrust.wrappers.RequestSecurityTokenResponse;
import org.picketlink.identity.federation.core.wstrust.wrappers.RequestSecurityTokenResponseCollection;
import org.picketlink.identity.federation.ws.trust.StatusType;
import org.picketlink.identity.federation.ws.trust.ValidateTargetType;
import org.w3c.dom.Element;

public class WSTrustClientTest {
	public static void main(String[] args) throws Exception {
		new WSTrustClientTest().testSTS();
	}

	public void testSTS() throws Exception {

		// create a dispatch object to invoke PicketLink STSs.
		Dispatch<Source> dispatch = createDispatch();

		// create a SAML token request message.
		RequestSecurityToken request = new RequestSecurityToken();
		request.setTokenType(URI.create(SAMLUtil.SAML2_TOKEN_TYPE));
		request.setRequestType(URI.create(WSTrustConstants.ISSUE_REQUEST));
		request.setContext("context");

		// send the token request to PicketLink STS and get the response.
		WSTrustJAXBFactory jaxbFactory = WSTrustJAXBFactory.getInstance();
		DOMSource requestSource = (DOMSource) jaxbFactory
				.marshallRequestSecurityToken(request);
		Source response = dispatch.invoke(requestSource);
		RequestSecurityTokenResponseCollection tokenResponse = (RequestSecurityTokenResponseCollection) jaxbFactory
				.parseRequestSecurityTokenResponse(response);
		System.out.println(tokenResponse);
		System.out.println("Response = " + response);
		Element data = (Element) tokenResponse
				.getRequestSecurityTokenResponses().get(0)
				.getRequestedSecurityToken().getAny();
		printAssertion(data);
		validate(dispatch, request, jaxbFactory, data);

	}

	private void validate(Dispatch<Source> dispatch,
			RequestSecurityToken request, WSTrustJAXBFactory jaxbFactory,
			Element assertion) throws WSTrustException {
		Source response;
		DOMSource requestSource;
		// let's validate the received SAML assertion.
		request.getAny().clear();
		request.setTokenType(URI.create(WSTrustConstants.STATUS_TYPE));
		request.setRequestType(URI.create(WSTrustConstants.VALIDATE_REQUEST));
		ValidateTargetType validateTarget = new ValidateTargetType();
		validateTarget.setAny(assertion);
		request.setValidateTarget(validateTarget);

		requestSource = (DOMSource) jaxbFactory
				.marshallRequestSecurityToken(request);

		response = dispatch.invoke(requestSource);
		RequestSecurityTokenResponseCollection responseCollection = (RequestSecurityTokenResponseCollection) jaxbFactory
				.parseRequestSecurityTokenResponse(response);
		RequestSecurityTokenResponse tokenResponse = responseCollection
				.getRequestSecurityTokenResponses().get(0);

		StatusType status = tokenResponse.getStatus();
		if (status != null) {
			String code = status.getCode();
			System.out.println("Signature is valid"
					+ WSTrustConstants.STATUS_CODE_INVALID.equals(code));

			System.out.println("\n\nSAMLV2.0 Assertion successfuly validated!");
			System.out.println("Validation status code: "
					+ tokenResponse.getStatus().getCode());
			System.out.println("Validation status reason: "
					+ tokenResponse.getStatus().getReason());
		} else
			System.out.println("\n\nFailed to validate SAMLV2.0 Assertion");
	}

	private Dispatch<Source> createDispatch() throws MalformedURLException,
			JAXBException {
		// PicketLink STS target information.
		String targetNS = "urn:picketlink:identity-federation:sts";
		QName serviceName = new QName(targetNS, "PicketLinkSTS");
		QName portName = new QName(targetNS, "PicketLinkSTSPort");
		URL endpointAddress = new URL(
				"http://localhost:8080/picketlink-sts/PicketLinkSTS");

		Service service = Service.create(serviceName);
		service.addPort(portName, SOAPBinding.SOAP11HTTP_BINDING,
				endpointAddress.toExternalForm());

		// create the dispatch, setting the client security configuration file.
		Dispatch<Source> dispatch = service.createDispatch(portName,
				Source.class, Mode.PAYLOAD);

		// add the username and password to the request context.
		Map<String, Object> reqContext = dispatch.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "admin");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "admin");

		return dispatch;
	}

	private void printAssertion(Element assertion) throws Exception {
		TransformerFactory tranFactory = TransformerFactory.newInstance();
		Transformer aTransformer = tranFactory.newTransformer();
		Source src = new DOMSource(assertion);
		Result dest = new StreamResult(System.out);
		aTransformer.transform(src, dest);
	}
}
